@@ -92,6 +92,14 @@ fn test_single_1_line() {
         .stdout_is_fixture("lorem_ipsum_1_line.expected");
 }
 
+#[test]
+fn test_single_1_line_presume_input_pipe() {
+    new_ucmd!()
+        .args(&["---presume-input-pipe", "-n", "1", INPUT])
+        .succeeds()
+        .stdout_is_fixture("lorem_ipsum_1_line.expected");
+}
+
 #[test]
 fn test_single_5_chars() {
     new_ucmd!()
@@ -149,6 +157,15 @@ fn test_zero_terminated_syntax_2() {
         .stdout_is("x\0y");
 }
 
+#[test]
+fn test_non_terminated_input() {
+    new_ucmd!()
+        .args(&["-n", "-1"])
+        .pipe_in("x\ny")
+        .succeeds()
+        .stdout_is("x\n");
+}
+
 #[test]
 fn test_zero_terminated_negative_lines() {
     new_ucmd!()
@@ -442,12 +459,19 @@ fn test_all_but_last_lines_large_file() {
     let scene = TestScenario::new(util_name!());
     let fixtures = &scene.fixtures;
     let seq_20000_file_name = "seq_20000";
+    let seq_20000_truncated_file_name = "seq_20000_truncated";
     let seq_1000_file_name = "seq_1000";
     scene
         .cmd("seq")
         .arg("20000")
         .set_stdout(fixtures.make_file(seq_20000_file_name))
         .succeeds();
+    // Create a file the same as seq_20000 except for the final terminating endline.
+    scene
+        .ucmd()
+        .args(&["-c", "-1", seq_20000_file_name])
+        .set_stdout(fixtures.make_file(seq_20000_truncated_file_name))
+        .succeeds();
     scene
         .cmd("seq")
         .arg("1000")
@@ -459,7 +483,7 @@ fn test_all_but_last_lines_large_file() {
         .ucmd()
         .args(&["-n", "-19000", seq_20000_file_name])
         .succeeds()
-        .stdout_only_fixture("seq_1000");
+        .stdout_only_fixture(seq_1000_file_name);
 
     scene
         .ucmd()
@@ -472,6 +496,25 @@ fn test_all_but_last_lines_large_file() {
         .args(&["-n", "-20001", seq_20000_file_name])
         .succeeds()
         .stdout_only_fixture("emptyfile.txt");
+
+    // Confirm correct behavior when the input file doesn't end with a newline.
+    scene
+        .ucmd()
+        .args(&["-n", "-19000", seq_20000_truncated_file_name])
+        .succeeds()
+        .stdout_only_fixture(seq_1000_file_name);
+
+    scene
+        .ucmd()
+        .args(&["-n", "-20000", seq_20000_truncated_file_name])
+        .succeeds()
+        .stdout_only_fixture("emptyfile.txt");
+
+    scene
+        .ucmd()
+        .args(&["-n", "-20001", seq_20000_truncated_file_name])
+        .succeeds()
+        .stdout_only_fixture("emptyfile.txt");
 }
 
 #[cfg(all(
@@ -728,7 +771,11 @@ fn test_read_backwards_bytes_proc_fs_modules() {
 
     let args = ["-c", "-1", "/proc/modules"];
     let result = ts.ucmd().args(&args).succeeds();
-    assert!(!result.stdout().is_empty());
+
+    // Only expect output if the file is not empty, e.g. it is empty in default WSL2.
+    if !ts.fixtures.read("/proc/modules").is_empty() {
+        assert!(!result.stdout().is_empty());
+    }
 }
 
 #[cfg(all(
@@ -744,7 +791,11 @@ fn test_read_backwards_lines_proc_fs_modules() {
 
     let args = ["--lines", "-1", "/proc/modules"];
     let result = ts.ucmd().args(&args).succeeds();
-    assert!(!result.stdout().is_empty());
+
+    // Only expect output if the file is not empty, e.g. it is empty in default WSL2.
+    if !ts.fixtures.read("/proc/modules").is_empty() {
+        assert!(!result.stdout().is_empty());
+    }
 }
 
 #[cfg(all(
@@ -757,7 +808,11 @@ fn test_read_backwards_lines_proc_fs_modules() {
 #[test]
 fn test_read_backwards_bytes_sys_kernel_profiling() {
     let ts = TestScenario::new(util_name!());
-
+    // in case the kernel was not built with profiling support, e.g. WSL
+    if !ts.fixtures.file_exists("/sys/kernel/profiling") {
+        println!("test skipped: /sys/kernel/profiling does not exist");
+        return;
+    }
     let args = ["-c", "-1", "/sys/kernel/profiling"];
     let result = ts.ucmd().args(&args).succeeds();
     let stdout_str = result.stdout_str();
@@ -803,3 +858,27 @@ fn test_write_to_dev_full() {
         }
     }
 }
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_head_non_utf8_paths() {
+    use std::ffi::OsStr;
+    use std::os::unix::ffi::OsStrExt;
+
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    // Create a test file with non-UTF-8 bytes in the name
+    let non_utf8_bytes = b"test_\xFF\xFE.txt";
+    let non_utf8_name = OsStr::from_bytes(non_utf8_bytes);
+
+    std::fs::write(at.plus(non_utf8_name), "line1\nline2\nline3\n").unwrap();
+
+    let result = scene.ucmd().arg(non_utf8_name).succeeds();
+
+    let output = result.stdout_str_lossy();
+    assert!(output.contains("line1"));
+    assert!(output.contains("line2"));
+    assert!(output.contains("line3"));
+}
+// Test that head handles non-UTF-8 file names without crashing
