@@ -426,6 +426,26 @@ fn test_mktemp_tmpdir() {
         .fails();
 }
 
+#[test]
+fn test_mktemp_empty_tmpdir() {
+    let scene = TestScenario::new(util_name!());
+    let pathname = scene.fixtures.as_string();
+
+    let result = scene
+        .ucmd()
+        .env(TMPDIR, &pathname)
+        .args(&["-p", ""])
+        .succeeds();
+    assert!(result.stdout_str().trim().starts_with(&pathname));
+
+    let result = scene
+        .ucmd()
+        .env(TMPDIR, &pathname)
+        .arg("--tmpdir=")
+        .succeeds();
+    assert!(result.stdout_str().trim().starts_with(&pathname));
+}
+
 #[test]
 fn test_mktemp_tmpdir_one_arg() {
     let scene = TestScenario::new(util_name!());
@@ -977,3 +997,66 @@ fn test_missing_short_tmpdir_flag() {
         .no_stdout()
         .stderr_contains("a value is required for '-p <DIR>' but none was supplied");
 }
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_template() {
+    use std::ffi::OsStr;
+    use std::os::unix::ffi::OsStrExt;
+
+    let ts = TestScenario::new(util_name!());
+
+    // Test that mktemp gracefully handles non-UTF-8 templates with an error instead of panicking
+    let template = OsStr::from_bytes(b"test_\xFF\xFE_XXXXXX");
+
+    ts.ucmd().arg(template).fails().stderr_contains("invalid");
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_tmpdir_path() {
+    use std::os::unix::ffi::OsStrExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    // Create a directory with non-UTF8 bytes
+    let dir_name = std::ffi::OsStr::from_bytes(b"test_dir_\xFF\xFE");
+    std::fs::create_dir(at.plus(dir_name)).unwrap();
+
+    // Test that mktemp can handle non-UTF8 directory paths with -p option
+    ucmd.arg("-p").arg(at.plus(dir_name)).succeeds();
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_tmpdir_long_option() {
+    use std::os::unix::ffi::OsStrExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    // Create a directory with non-UTF8 bytes
+    let dir_name = std::ffi::OsStr::from_bytes(b"test_dir_\xFF\xFE");
+    std::fs::create_dir(at.plus(dir_name)).unwrap();
+
+    // Test that mktemp can handle non-UTF8 directory paths with --tmpdir option
+    // Note: Due to test framework limitations with non-UTF8 arguments and --tmpdir= syntax,
+    // we'll test a more limited scenario that still validates non-UTF8 path handling
+    ucmd.arg("-p")
+        .arg(at.plus(dir_name))
+        .arg("tmpXXXXXX")
+        .succeeds();
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_tmpdir_directory_creation() {
+    use std::os::unix::ffi::OsStrExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    // Create a directory with non-UTF8 bytes
+    let dir_name = std::ffi::OsStr::from_bytes(b"test_dir_\xFF\xFE");
+    std::fs::create_dir(at.plus(dir_name)).unwrap();
+
+    // Test directory creation (-d flag) with non-UTF8 directory paths
+    // We can't easily verify the exact output path because of UTF8 conversion issues,
+    // but we can verify the command succeeds
+    ucmd.arg("-d").arg("-p").arg(at.plus(dir_name)).succeeds();
+}
