@@ -3,19 +3,35 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 //
-// spell-checker:ignore binvalid finvalid hinvalid iinvalid linvalid nabcabc nabcabcabc ninvalid vinvalid winvalid
-use crate::common::util::TestScenario;
+// spell-checker:ignore binvalid finvalid hinvalid iinvalid linvalid nabcabc nabcabcabc ninvalid vinvalid winvalid dabc näää
+use uutests::{at_and_ucmd, new_ucmd, util::TestScenario, util_name};
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_paths() {
+    use std::os::unix::ffi::OsStringExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let filename = std::ffi::OsString::from_vec(vec![0xFF, 0xFE]);
+    std::fs::write(at.plus(&filename), b"line 1\nline 2\nline 3\n").unwrap();
+
+    ucmd.arg(&filename)
+        .succeeds()
+        .stdout_contains("1\t")
+        .stdout_contains("2\t")
+        .stdout_contains("3\t");
+}
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
 fn test_stdin_no_newline() {
     new_ucmd!()
         .pipe_in("No Newline")
-        .run()
+        .succeeds()
         .stdout_is("     1\tNo Newline\n");
 }
 
@@ -24,7 +40,7 @@ fn test_stdin_newline() {
     new_ucmd!()
         .args(&["-s", "-", "-w", "1"])
         .pipe_in("Line One\nLine Two\n")
-        .run()
+        .succeeds()
         .stdout_is("1-Line One\n2-Line Two\n");
 }
 
@@ -32,7 +48,7 @@ fn test_stdin_newline() {
 fn test_padding_without_overflow() {
     new_ucmd!()
         .args(&["-i", "1000", "-s", "x", "-n", "rz", "simple.txt"])
-        .run()
+        .succeeds()
         .stdout_is(
             "000001xL1\n001001xL2\n002001xL3\n003001xL4\n004001xL5\n005001xL6\n006001xL7\n0070\
              01xL8\n008001xL9\n009001xL10\n010001xL11\n011001xL12\n012001xL13\n013001xL14\n014\
@@ -44,7 +60,7 @@ fn test_padding_without_overflow() {
 fn test_padding_with_overflow() {
     new_ucmd!()
         .args(&["-i", "1000", "-s", "x", "-n", "rz", "-w", "4", "simple.txt"])
-        .run()
+        .succeeds()
         .stdout_is(
             "0001xL1\n1001xL2\n2001xL3\n3001xL4\n4001xL5\n5001xL6\n6001xL7\n7001xL8\n8001xL9\n\
              9001xL10\n10001xL11\n11001xL12\n12001xL13\n13001xL14\n14001xL15\n",
@@ -73,7 +89,7 @@ fn test_sections_and_styles() {
             .args(&[
                 "-s", "|", "-n", "ln", "-w", "3", "-b", "a", "-l", "5", fixture,
             ])
-            .run()
+            .succeeds()
             .stdout_is(output);
     }
     // spell-checker:enable
@@ -190,6 +206,28 @@ fn test_number_separator() {
     }
 }
 
+#[test]
+#[cfg(target_os = "linux")]
+fn test_number_separator_non_utf8() {
+    use std::{
+        ffi::{OsStr, OsString},
+        os::unix::ffi::{OsStrExt, OsStringExt},
+    };
+
+    let separator_bytes = [0xFF, 0xFE];
+    let mut v = b"--number-separator=".to_vec();
+    v.extend_from_slice(&separator_bytes);
+
+    let arg = OsString::from_vec(v);
+    let separator = OsStr::from_bytes(&separator_bytes);
+
+    new_ucmd!()
+        .arg(arg)
+        .pipe_in("test")
+        .succeeds()
+        .stdout_is(format!("     1{}test\n", separator.to_string_lossy()));
+}
+
 #[test]
 fn test_starting_line_number() {
     for arg in ["-v10", "--starting-line-number=10"] {
@@ -285,6 +323,25 @@ fn test_join_blank_lines() {
     }
 }
 
+#[test]
+fn test_join_blank_lines_zero() {
+    for arg in ["-l0", "--join-blank-lines=0"] {
+        new_ucmd!()
+            .arg(arg)
+            .arg("--body-numbering=a")
+            .pipe_in("\n\n\n\n\n\n")
+            .succeeds()
+            .stdout_is(concat!(
+                "     1\t\n",
+                "     2\t\n",
+                "     3\t\n",
+                "     4\t\n",
+                "     5\t\n",
+                "     6\t\n",
+            ));
+    }
+}
+
 #[test]
 fn test_join_blank_lines_multiple_files() {
     let scene = TestScenario::new(util_name!());
@@ -310,15 +367,6 @@ fn test_join_blank_lines_multiple_files() {
     }
 }
 
-#[test]
-fn test_join_blank_lines_zero() {
-    for arg in ["-l0", "--join-blank-lines=0"] {
-        new_ucmd!().arg(arg).fails().stderr_contains(
-            "Invalid line number of blank lines: ‘0’: Numerical result out of range",
-        );
-    }
-}
-
 #[test]
 fn test_invalid_join_blank_lines() {
     for arg in ["-linvalid", "--join-blank-lines=invalid"] {
@@ -634,3 +682,36 @@ fn test_empty_section_delimiter() {
             .stdout_is("     1\ta\n       \n     2\tb\n");
     }
 }
+
+#[test]
+fn test_directory_as_input() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir = "dir";
+    let file = "file";
+    let content = "aaa";
+
+    at.mkdir(dir);
+    at.write(file, content);
+
+    ucmd.arg(dir)
+        .arg(file)
+        .fails()
+        .stderr_is(format!("nl: {dir}: Is a directory\n"))
+        .stdout_contains(content);
+}
+
+#[test]
+fn test_file_with_non_utf8_content() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let filename = "file";
+    let content: &[u8] = b"a\n\xFF\xFE\nb";
+    let invalid_utf8: &[u8] = b"\xFF\xFE";
+
+    at.write_bytes(filename, content);
+
+    ucmd.arg(filename).succeeds().stdout_is(format!(
+        "     1\ta\n     2\t{}\n     3\tb\n",
+        String::from_utf8_lossy(invalid_utf8)
+    ));
+}
