@@ -2,15 +2,26 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-#![allow(unused_imports)]
-mod common;
-
-use common::util::TestScenario;
+use uutests::util::TestScenario;
 
 #[cfg(unix)]
 use std::os::unix::fs::symlink as symlink_file;
-#[cfg(windows)]
-use std::os::windows::fs::symlink_file;
+
+use std::env;
+pub const TESTS_BINARY: &str = env!("CARGO_BIN_EXE_coreutils");
+
+// Set the environment variable for any tests
+
+// Use the ctor attribute to run this function before any tests
+#[ctor::ctor]
+fn init() {
+    // No need for unsafe here
+    unsafe {
+        std::env::set_var("UUTESTS_BINARY_PATH", TESTS_BINARY);
+    }
+    // Print for debugging
+    eprintln!("Setting UUTESTS_BINARY_PATH={TESTS_BINARY}");
+}
 
 #[test]
 #[cfg(feature = "ls")]
@@ -18,36 +29,25 @@ fn execution_phrase_double() {
     use std::process::Command;
 
     let scenario = TestScenario::new("ls");
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
     let output = Command::new(&scenario.bin_path)
         .arg("ls")
         .arg("--some-invalid-arg")
+        .env("LANG", "en_US.UTF-8")
         .output()
         .unwrap();
-    assert!(String::from_utf8(output.stderr)
-        .unwrap()
-        .contains(&format!("Usage: {} ls", scenario.bin_path.display(),)));
-}
-
-#[test]
-#[cfg(feature = "ls")]
-#[cfg(any(unix, windows))]
-fn execution_phrase_single() {
-    use std::process::Command;
-
-    let scenario = TestScenario::new("ls");
-    symlink_file(scenario.bin_path, scenario.fixtures.plus("uu-ls")).unwrap();
-    let output = Command::new(scenario.fixtures.plus("uu-ls"))
-        .arg("--some-invalid-arg")
-        .output()
-        .unwrap();
-    dbg!(String::from_utf8(output.stderr.clone()).unwrap());
-    assert!(String::from_utf8(output.stderr).unwrap().contains(&format!(
-        "Usage: {}",
-        scenario.fixtures.plus("uu-ls").display()
-    )));
+    assert!(
+        String::from_utf8(output.stderr)
+            .unwrap()
+            .contains(&"Usage: ls".to_string())
+    );
 }
 
 #[test]
+#[ignore = "Test assumes error upon non-UTF8"]
 #[cfg(feature = "sort")]
 fn util_name_double() {
     use std::{
@@ -56,7 +56,11 @@ fn util_name_double() {
     };
 
     let scenario = TestScenario::new("sort");
-    let mut child = Command::new(scenario.bin_path)
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+    let mut child = Command::new(&scenario.bin_path)
         .arg("sort")
         .stdin(Stdio::piped())
         .stderr(Stdio::piped())
@@ -69,8 +73,9 @@ fn util_name_double() {
 }
 
 #[test]
+#[ignore = "Test assumes error upon non-UTF8"]
 #[cfg(feature = "sort")]
-#[cfg(any(unix, windows))]
+#[cfg(unix)]
 fn util_name_single() {
     use std::{
         io::Write,
@@ -78,7 +83,12 @@ fn util_name_single() {
     };
 
     let scenario = TestScenario::new("sort");
-    symlink_file(scenario.bin_path, scenario.fixtures.plus("uu-sort")).unwrap();
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+
+    symlink_file(&scenario.bin_path, scenario.fixtures.plus("uu-sort")).unwrap();
     let mut child = Command::new(scenario.fixtures.plus("uu-sort"))
         .stdin(Stdio::piped())
         .stderr(Stdio::piped())
@@ -94,15 +104,16 @@ fn util_name_single() {
 }
 
 #[test]
-#[cfg(any(unix, windows))]
+#[cfg(unix)]
 fn util_invalid_name_help() {
-    use std::{
-        io::Write,
-        process::{Command, Stdio},
-    };
+    use std::process::{Command, Stdio};
 
     let scenario = TestScenario::new("invalid_name");
-    symlink_file(scenario.bin_path, scenario.fixtures.plus("invalid_name")).unwrap();
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+    symlink_file(&scenario.bin_path, scenario.fixtures.plus("invalid_name")).unwrap();
     let child = Command::new(scenario.fixtures.plus("invalid_name"))
         .arg("--help")
         .stdin(Stdio::piped())
@@ -130,15 +141,18 @@ fn util_non_utf8_name_help() {
     // Make sure we don't crash even if the util name is invalid UTF-8.
     use std::{
         ffi::OsStr,
-        io::Write,
         os::unix::ffi::OsStrExt,
-        path::Path,
         process::{Command, Stdio},
     };
 
     let scenario = TestScenario::new("invalid_name");
     let non_utf8_path = scenario.fixtures.plus(OsStr::from_bytes(b"\xff"));
-    symlink_file(scenario.bin_path, &non_utf8_path).unwrap();
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+
+    symlink_file(&scenario.bin_path, &non_utf8_path).unwrap();
     let child = Command::new(&non_utf8_path)
         .arg("--help")
         .stdin(Stdio::piped())
@@ -158,15 +172,17 @@ fn util_non_utf8_name_help() {
 }
 
 #[test]
-#[cfg(any(unix, windows))]
+#[cfg(unix)]
 fn util_invalid_name_invalid_command() {
-    use std::{
-        io::Write,
-        process::{Command, Stdio},
-    };
+    use std::process::{Command, Stdio};
 
     let scenario = TestScenario::new("invalid_name");
-    symlink_file(scenario.bin_path, scenario.fixtures.plus("invalid_name")).unwrap();
+    symlink_file(&scenario.bin_path, scenario.fixtures.plus("invalid_name")).unwrap();
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+
     let child = Command::new(scenario.fixtures.plus("invalid_name"))
         .arg("definitely_invalid")
         .stdin(Stdio::piped())
@@ -186,13 +202,15 @@ fn util_invalid_name_invalid_command() {
 #[test]
 #[cfg(feature = "true")]
 fn util_completion() {
-    use std::{
-        io::Write,
-        process::{Command, Stdio},
-    };
+    use std::process::{Command, Stdio};
 
     let scenario = TestScenario::new("completion");
-    let child = Command::new(scenario.bin_path)
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+
+    let child = Command::new(&scenario.bin_path)
         .arg("completion")
         .arg("true")
         .arg("powershell")
@@ -214,13 +232,15 @@ fn util_completion() {
 #[test]
 #[cfg(feature = "true")]
 fn util_manpage() {
-    use std::{
-        io::Write,
-        process::{Command, Stdio},
-    };
+    use std::process::{Command, Stdio};
 
     let scenario = TestScenario::new("completion");
-    let child = Command::new(scenario.bin_path)
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+
+    let child = Command::new(&scenario.bin_path)
         .arg("manpage")
         .arg("true")
         .stdin(Stdio::piped())
@@ -234,3 +254,29 @@ fn util_manpage() {
     let output_str = String::from_utf8(output.stdout).unwrap();
     assert!(output_str.contains("\n.TH true 1 "), "{output_str:?}");
 }
+
+#[test]
+fn util_version() {
+    use std::process::{Command, Stdio};
+
+    let scenario = TestScenario::new("--version");
+    if !scenario.bin_path.exists() {
+        println!("Skipping test: Binary not found at {:?}", scenario.bin_path);
+        return;
+    }
+    for arg in ["-V", "--version"] {
+        let child = Command::new(&scenario.bin_path)
+            .arg(arg)
+            .stdin(Stdio::piped())
+            .stdout(Stdio::piped())
+            .stderr(Stdio::piped())
+            .spawn()
+            .unwrap();
+        let output = child.wait_with_output().unwrap();
+        assert_eq!(output.status.code(), Some(0));
+        assert_eq!(output.stderr, b"");
+        let output_str = String::from_utf8(output.stdout).unwrap();
+        let ver = std::env::var("CARGO_PKG_VERSION").unwrap();
+        assert_eq!(format!("coreutils {ver} (multi-call binary)\n"), output_str);
+    }
+}
